## 26

install.packages('tm')

library(tm)

## 27

install.packages('twitteR')

library(twitteR)

con <- url("http://www.rdatamining.com/data/rdmTweets.RData")

load(con)

(nDocs <- length(rdmTweets))
rdmTweets[11:15]

## 29

class(rdmTweets)
typeof(rdmTweets)
df <- twListToDF(rdmTweets)
dim(df)
dimnames(df)

## 30

myCorpus <- Corpus(VectorSource(df$text))
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus[[i]], width=73)))
}

myCorpus <- tm_map(myCorpus, content_transformer(tolower))
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus[[i]], width=73)))
}

## 31

myCorpus2 <- tm_map(myCorpus, removePunctuation)
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus2[[i]], width=73)))
}

toSpace <- content_transformer(function (x, pattern) gsub(pattern, " ", x))
myCorpus3 <- tm_map(myCorpus2, toSpace, ",")

myCorpus4 <- tm_map(myCorpus3, removeNumbers)
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus4[[i]], width=73)))
}

## 32

removeURL <- content_transformer(function(x) gsub("http[[:alnum:]]*", "", x))
myCorpus5 <- tm_map(myCorpus4, removeURL)
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus5[[i]], width=73)))
}

## 34

myStopwords <- c(stopwords('english'), "available", "via")
stopwords('english')
length(stopwords('en'))
myStopwords <- c(stopwords('english'), "available", "via", "r")
myStopwords
length(myStopwords)

## 35

idx <- which(myStopwords %in% c('r', 'big'))
idx
myStopwords <- myStopwords[-idx]
length(myStopwords)
myCorpus6 <- tm_map(myCorpus5, removeWords, myStopwords)
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus6[[i]], width=73)))
}

## 36

install.packages('SnowballC')
library(SnowballC)
myCorpusCopy <- myCorpus6
myCorpus7 <- tm_map(myCorpus6, stemDocument)
for (i in 11:15) {
  cat(paste("[[", i, "]] ", sep=""))
  writeLines((strwrap(myCorpus7[[i]], width=73)))
}

## 37
install.packages('KoNLP')
library(KoNLP)
extractNoun('ʿ Ű εԴϴ.')

## 39

myTdm <- TermDocumentMatrix(myCorpus7, control = list(wordLengths = c(1, Inf)))
myTdm
idx <- which(dimnames(myTdm)$Terms == 'r')
inspect(myTdm[idx+(0:5), 101:110])
rownames(myTdm)

## 40

myTdm2 <- TermDocumentMatrix(myCorpus7, control = list(wordLengths = c(1, Inf), weighting = weightTfIdf))
myTdm2
idx <- which(dimnames(myTdm2)$Terms == 'r')
inspect(myTdm2[idx+(0:5), 101:110])

## 43

findFreqTerms(myTdm, lowfreq = 10)
termFrequency <- rowSums(as.matrix(myTdm))
termFrequency <- subset(termFrequency, termFrequency >= 10)
#install.packages('ggplot2')
library(ggplot2)
df1 <- data.frame(x = names(termFrequency), y = termFrequency)
p <- ggplot(df1, aes(x, y)) + geom_bar(stat="identity") + coord_flip()
print(p)

## 44

barplot(termFrequency, las=2)

## 45
#install.packages('wordcloud')
library(wordcloud)
m <- as.matrix(myTdm)
wordFreq <- sort(rowSums(m), decreasing = TRUE)
set.seed(375)
grayLevels <- gray((wordFreq + 10) / (max(wordFreq) + 10))
wordcloud(words = names(wordFreq), freq = wordFreq, min.freq = 3, random.order = F, colors = grayLevels)


## 46

#install.packages('igraph')
findAssocs(myTdm, 'r', 0.25)
findAssocs(myTdm, 'mine', 0.25)
assoc <- findAssocs(myTdm, 'mine', 0.25)
rulemat <- data.frame('mine', names(assoc$mine), assoc$mine)
library(igraph)
ruleg <- graph.edgelist(as.matrix(rulemat[,1:2]), directed = F)
plot.igraph(ruleg, vertex.label=V(ruleg)$name, vertex.label.cex=1.5, vertex.size = 25, margin = 0.4)

## 47

#install.packages('corrplot')
library(corrplot)
titles <- c('mine', names(assoc$mine))
m3 <- as.matrix(myTdm)
m3 <- m3[which(rownames(m3) %in% titles),]
corr_rule <- cor(t(m3))
corrplot(type='lower', tl.cex = .6, corr=corr_rule, tl.col = 'grey20', method = 'circle', order='FPC', addtextlable = 'ld', outline = TRUE)

## 56

myTdm2 <- removeSparseTerms(myTdm, sparse = 0.95)
m2 <- as.matrix(myTdm2)
distMatrix <- dist(scale(m2))
fit <- hclust(distMatrix, method = 'ward.D')
plot(fit)
rect.hclust(fit, k = 10)
(groups <- cutree(fit, k=10))

## 59

m3 <- t(m2)
set.seed(122)
k <- 8
kmeansResult <- kmeans(m3, k)
round(kmeansResult$centers, digits = 3)

## 60

for (i in 1:k) {
  cat(paste("cluster", i, ": ", sep=""))
  s <- sort(kmeansResult$centers[i, ], decreasing = T)
  cat(names(s)[1:3], "\n")
  print(rdmTweets[which(kmeansResult$cluster == i)])
}

## 61

#install.packages('fpc')
library(fpc)
pamResult <- pamk(m3, metric = 'manhattan')
(k <- pamResult$nc)
pamResult <- pamResult$pamobject
for (i in 1:k) {
  cat(paste("cluster", i, ": ", sep=""))
  cat(colnames(pamResult$medoids)[which(pamResult$medoids[i,] == 1)], "\n")
  #print(rdmTweets[pamResult$clustering == i])
}

## 62

pamResult$medoids[7, ]
pamResult$clusinfo
layout(matrix(c(1,2), 2, 1))
plot(pamResult, color = F, labels = 4, lines = 0, cex = .8, col.clus = 1, col.p = pamResult$clustering)
layout(matrix(1))

## 64

pamResult2 <- pamk(m3, krang = 2:8, metric = 'manhattan')
pamResult2 <- pamResult2$pamobject
layout(matrix(c(1,2), 2, 1))
plot(pamResult2, color = F, labels = 4, lines = 0, cex = .8, col.clus = 1, col.p = pamResult2$clustering)
layout(matrix(1))

## 66

#install.packages("devtools")
require(devtools)
#install_url("http://cran.r-project.org/src/contrib/Archive/sentiment/sentiment_0.2.tar.gz")
require(sentiment)
x <- c()
for (i in 1:length(myCorpus7)) {
  x <- c(x, strwrap(myCorpus7[[i]]))
}
sent <- classify_emotion(x, algorithm="bayes", verbose = TRUE)

## 67

data <- read.csv(system.file("data/emotions.csv.gz", package = 'sentiment'))
data(emotions)
emotions[1:5, 1]
sent[8:10, ]
for(i in which(!is.na(sent[,7]))) {
  cat(paste("[[", i, "]] ", sep=""))
  cat(paste(" - ", sent[i,7], " - ", sep = ""))
  writeLines(strwrap(myCorpus7[[i]], width = 73))
}

## 71

m2[m2 >= 1] <- 1
myTm <- m2 %*% t(m2)
myTm[5:10, 5:10]
library(igraph)
g <- graph.adjacency(myTm, weighted = T, mode = 'undirected')
g <- simplify(g)
V(g)$label <- V(g)$name
V(g)$degree <- degree(g)
set.seed(3952)
layout1 <- layout.fruchterman.reingold(g)
plot(g, layout = layout1)


## 72 

plot(g, layout = layout.kamada.kawai)
tkplot(g, layout = layout.kamada.kawai)
pdf("term-network.pdf")
plot(g, layout = layout.fruchterman.reingold)
dev.off()

## 73

V(g)$label.cex <- 2.2 * V(g)$degree / max(V(g)$degree) + .2
V(g)$label.color <- rgb(0, 0, .2, .8)
V(g)$frame.color <- NA
egam <- (log(E(g)$weight) + .4) / max(log(E(g)$weight) + .4)
E(g)$color <- rgb(.5, .5, 0, egam)
E(g)$width <- egam
plot(g, layout=layout1)

## 74

idx <- which(dimnames(m2)$Terms %in% c("r","data", "mine"))
M <- m2[-idx,]
tweetMatrix <- t(M) %*% M
library(igraph)
g <- graph.adjacency(tweetMatrix, weighted = T, mode = 'undirected')
V(g)$degree <- degree(g)
g <- simplify(g)
V(g)$label <- V(g)$name
V(g)$label.cex <- 1
V(g)$label.color <- rgb(.4, 0, 0, .7)
V(g)$size <- 2
V(g)$frame.color <- NA
barplot(table(V(g)$degree))

## 75

idx <- V(g)$degree == 0
V(g)$label.color[idx] <- rgb(0, 0, .3, .7)
V(g)$label[idx] <- paste(V(g)$name[idx], substr(df$text[idx], 1, 20), sep = ": ")
egam <- (log(E(g)$weight) + .2) / max(log(E(g)$weight) + .2)
E(g)$color <- rgb(.5, .5, 0, egam)
E(g)$width <- egam
set.seed(3152)
layout2 <- layout.fruchterman.reingold(g)
plot(g, layout = layout2)

## 76

g2 <- delete.vertices(g, V(g)[degree(g)==0])
plot(g2, layout = layout.fruchterman.reingold)

## 77

g3 <- delete.edges(g, E(g)[E(g)$weight <= 1])
g3 <- delete.vertices(g3, V(g3)[degree(g3) == 0])
plot(g3, layout = layout.fruchterman.reingold)

## 78

df$text[c(7,12,6,9,8,3,4)]
df$text[c(4,5, 154,71)]
df$text[c(4,33,94,29,18,92)]
df$text[c(112,129,119,105,108,136)]

## 79

g <- graph.incidence(m2, mode = c('all'))
nTerms <- nrow(M)
nDocs <- ncol(M)
idx.terms <- 1:nTerms
idx.docs <- (nTerms + 1):(nTerms + nDocs)
V(g)$degree <- degree(g)
V(g)$color[idx.terms] <- rgb(0, 1, 0, .5)
V(g)$size[idx.terms] <- 6
V(g)$color[idx.docs] <- rgb(1, 0, 0, .4)
V(g)$size[idx.docs] <- 4
V(g)$frame.color <- NA
V(g)$label <- V(g)$name
V(g)$label.color <- rgb(0, 0, 0, .5)
V(g)$label.cex <- 1.4 * V(g)$degree / max(V(g)$degree) + 1
E(g)$width <- 3
E(g)$color <- rgb(.5, .5, 0, .3)
set.seed(958)
plot(g, layout = layout.fruchterman.reingold)

## 80

V(g)[nei("r")]
V(g)[neighborhood(g, order = 1, "r")[[1]]]
(rdmVertices <- V(g)[nei("r") & nei("data") & nei("mine")])
df$text[as.numeric(rdmVertices$label)]

## 81

idx <- which(V(g)$name %in% c("r", "data", "mine"))
g2 <- delete.vertices(g, V(g)[idx])
g2 <- delete.vertices(g2, V(g2)[degree(g2) == 0])
set.seed(209)
plot(g2, layout = layout.fruchterman.reingold)
